@@ -13,10 +13,11 @@ filterid_dict = {
 
 
 def return_filename(filename):
-    try:
-        filename = filename.decode()
-    except AttributeError:
-        pass
+    if type(filename) != str:
+        try:
+            filename = filename.decode()
+        except AttributeError:
+            pass
 
     if filename is None:
         raise FileNotFoundError(filename)
@@ -52,3 +53,24 @@ def _return_util_filename(filename, extension):
               "https://github.com/MichaelMedford/zort for more details.")
         print('******')
         raise FileNotFoundError(util_filename)
+
+
+def sortsplit(array, size):
+    """Returns array split into different lists in round-robin order.
+
+    A fun way to split up a list in round-robin order into separate lists.
+
+    Args:
+        array : list
+            List of items to be split into separate lists.
+        size : type
+            Number of lists in which array will be split into.
+
+    Returns:
+        train : list
+            A list of lists, each a piece of the provided array
+
+    """
+    # Have some fun thinking it through! A pen and paper might help :)
+    return [[array[i] for i in range(j, len(array), size)]
+            for j in range(size)]
